#!/bin/sh

. /etc/init.d/common

fsck_failed ()
{
	echo "$2"
	retval="$1"
	echo $"Exit status of fsck is $retval."
	echo $"The system will be rebooted when you log out (Ctrl+D)."
	export PS1='# '
	/sbin/sulogin
	umount -a -d -t noproc,nosysfs,nodevfs,nodevpts,noramfs
	mount -n -o remount,ro /
	sync
	/sbin/reboot -d -f -i
}

# mount /proc
mount -n -t proc none /proc
mount -n -t binfmt_misc none /proc/sys/fs/binfmt_misc

# mount /sys
mount -n -t sysfs none /sys

# mount /media
mount -n -t ramfs none /media
chgrp media /media
chmod 750 /media

# As of kernel 2.6.15, "drivers" events cannot be replayed,
# so we have to take care of it:
if [ -d /proc/bus/usb -a ! -f /proc/bus/usb/devices ]; then
	mount -n -t usbfs none /proc/bus/usb
fi

# start udev
/sbin/udevd -d
/sbin/udevtrigger
/sbin/udevsettle
rootdev="$(/bin/getdev /)"
ln -snf "${rootdev#/dev/}" /dev/root

exec 0<>/dev/console
exec 1<>/dev/console
exec 2<>/dev/console

# turn on UTF-8 output mode
echo -ne '\e%G'

# set console font
if [ -f /etc/sysconfig/consolefont ]; then
	consolefont="$(</etc/sysconfig/consolefont)"
else
	consolefont=$"consolefont|lat1-16"
	consolefont="${consolefont#consolefont|}"
fi
if [ -f /etc/sysconfig/consolemap ]; then
	consolemap="$(</etc/sysconfig/consolemap)"
else
	consolemap=$"consolemap|8859-1"
	consolemap="${consolemap#consolemap|}"
fi
setfont "$consolefont" -m "$consolemap"

# set keyboard layout
kbd_mode -u
if [ -f /etc/sysconfig/consolelayout ]; then
	consolelayout="$(</etc/sysconfig/consolelayout)"
else
	consolelayout=$"consolelayout|us"
	consolelayout="${consolelayout#consolelayout|}"
fi
loadkeys -q $consolelayout

# clear screen carefully
if [ -z "$UHU_DEBUG" ]; then
	scrollout
fi

# welcome banner
echo $'\e[01;32mUHU-Linux '"$(</etc/uhu-release)"$'\e[0m\n'

function probemod() {
	echo -en "\t$@"
	if modprobe -qqq "$@"; then
		echo $'\e[74G\e[01;32m[ OK ]\e[0m'
	else
		echo
	fi
}

echo $"Loading kernel modules"
if [ ! -d /proc/acpi -a ! -d /sys/firmware/acpi -a ! -d /sys/devices/system/acpi ]; then
	probemod apm
fi
if [ -f /proc/driver/nvram ] && grep ^Floppy /proc/driver/nvram | grep -qv 'none$'; then
	probemod floppy
fi
probemod capability
probemod loop

# wait for nodes the above modules might have created
/sbin/udevsettle

# activating swap, first try
echo -n $"Activating virtual memory"
swap_initialized=0
swapon -a && { echo "$OK"; swap_initialized=1; } || echo "$ERR";

# check /
rootfs_mode=ro
while read part mp type mode rest; do
	if [ "$mp" = "/" ]; then
		if [ "${mode/rw/}" != "${mode}" ]; then
			rootfs_mode=rw
		else
			rootfs_mode=ro
		fi
	fi
done < /proc/mounts
if [ "$rootfs_mode" = ro ]; then
	echo $"Checking root partition"
	FSCK_QUIET_IF_CLEAN=1 fsck -T -C -a "$(getdev /)"
	retval=$?
	case $? in
		0)	echo "$OK";;
		1)	echo "$DONE";;
		*)	fsck_failed $? $"The root file system is damaged, please repair it manually!";;
	esac
fi

# making / writeable
echo -n $"Remounting root file system"
mount -n -o remount,rw / && echo "$OK" || echo "$ERR"
if [ ! -L /etc/mtab ]; then
	cat /proc/mounts >/etc/mtab
fi
echo $"System boot in progress." > /etc/nologin

# Start RAID arrays
if [ -d /dev/disk/by-type/linux_raid_member -a -x /sbin/mdadm ]; then
	mkdir -p /dev/md
	/sbin/mdadm -A -s -c partitions --auto=yes --quiet --auto-update-homehost --homehost=localhost
	rmdir --ignore-fail-on-non-empty /dev/md
fi

# Start LVM volumes
if [ -d /dev/disk/by-type/linux_raid_member -o -d /dev/disk/by-type/LVM2_member -a -x /sbin/vgscan -a -x /sbin/vgchange ]; then
	/sbin/vgscan --quiet --ignorelockingfailure
	/sbin/vgchange --quiet --ignorelockingfailure -a y
fi

# check local partitions
echo $"Checking other local file systems"
FSCK_QUIET_IF_CLEAN=1 fsck -T -C -a -A -R
case $? in
	0)	echo "$OK";;
	1)	echo "$DONE";;
	*)	fsck_failed $? $"One of the file systems is damaged, please repair it manually!";;
esac

# mount all
echo -n $"Mounting other local file systems"
mount -a -t nonfs,nosmbfs,noncp,noafs,nocifs,nodavfs && echo "$OK" || echo "$ERR"

# hostname
echo -n $"Setting hostname: "
if [ -s /etc/hostname ]; then
	cat /etc/hostname
else
	echo "localhost"
fi >/proc/sys/kernel/hostname
echo -n "$(</proc/sys/kernel/hostname)"
echo "$OK"

# activating swap, second try
if [ $swap_initialized = 0 ]; then
	echo -n $"Activating virtual memory"
	swapon -a && echo "$OK" || echo "$ERR"
fi

# other parameters
if [ -s /etc/sysctl.conf ]; then
	echo -n $"Setting other system parameters"
	sysctl -p >/dev/null 2>&1 && echo "$OK" || echo "$ERR"
fi

# Cleanup
echo -n $"Removing temporary files"
#  /tmp
find /tmp/ -xdev -mindepth 1 -maxdepth 1 \
  ! \( -name lost+found   -uid 0 \) \
  ! \( -name quota.user   -uid 0 \) \
  ! \( -name quota.group  -uid 0 \) \
  ! \( -name aquota.user  -uid 0 \) \
  ! \( -name aquota.group -uid 0 \) \
  ! \( -name .journal     -uid 0 \) \
  ! \( -name .X11-unix    -uid 0 \) \
  ! \( -name .ICE-unix    -uid 0 \) \
  -print0 | xargs -r0 rm -rf --
find /tmp/.X11-unix/ /tmp/.ICE-unix/ -xdev -mindepth 1 -maxdepth 1 \
  -print0 | xargs -r0 rm -rf --
# /var/lock
find /var/lock/ -xdev -mindepth 1 ! -type d -print0 | xargs -r0 rm -f --
# /var/run
find /var/run/  -xdev -mindepth 1 ! -type d -print0 | xargs -r0 rm -f --
# utmp
>/var/run/utmp
echo "$OK"

# date and time
result="$OK"
echo -n $"Setting system clock: "
hwclock-wrapper --hctosys || result="$ERR"
echo -n "$(date)"
echo "$result"

# setup loopback network
ifconfig lo 127.0.0.1 netmask 255.0.0.0

exit 0
